/* Simple optimization example. See README for details. */

#include <cmath>
#include <cstdio>
#include <cstring>
#include <cstdlib>

#include "minuitfcn.h"
#include "cfortran.h"
#include "cminuit.h"

PROTOCCALLSFSUB3(MNINIT,mninit,PINT,PINT,PINT);
PROTOCCALLSFSUB7(MNPARM,mnparm,INT,STRING,DOUBLE,DOUBLE,PDOUBLE,PDOUBLE,PINT);
PROTOCCALLSFSUB4(MNCOMD,mncomd,ROUTINE,STRING,PINT,ROUTINE);
PROTOCCALLSFSUB8(MNCONT,mncont,ROUTINE,INT,INT,INT,PDOUBLE,PDOUBLE,PINT,ROUTINE);

#define MAXLINE 256

typedef struct {
  char *name;
  double value;
  double error;
  double min;
  double max;
} par_t;

par_t pars[] = {
  { "x", 0., 5, 0., 0. },
  { "y", 0., 1, 0., 0. },
  { "z", 0., 1, 0., 0. },
};

class CMNInfo{
public:
  static double x0,y0,z0;
  static void fcn(int npar, double* grad, double* fcnval, double* xval, int iflag, void* futil);
};
double CMNInfo::x0=1.0;
double CMNInfo::y0=2.0;
double CMNInfo::z0=3.0;

CMNInfo *MNInfo_ptr;

void CMNInfo::fcn(int npar, double* grad, double* fcnval, double* xval, int iflag, void* futil)
{
  *fcnval = pow(xval[0]-x0,2)  +10*pow(xval[0]-x0,4) +pow(xval[1]-y0,2) + pow(xval[2]-z0,2);
  //*fcnval +=20*(xval[0]-1.0)*xval[1];
}

void fcn (int npar, double* grad, double* fcnval, double* xval, int iflag, void* futil)
{
  double bj=1.0;
  *fcnval = pow(xval[0]-bj,2)  +10*pow(xval[0]-bj,4) +pow(xval[1],2) + pow(xval[2],2);
  //*fcnval +=20*(xval[0]-1.0)*xval[1];
}

int main (int argc, char *argv[]){
  CMNInfo mninfo;
  int i, dummy;
  char *command = NULL;
  double *xcontour,*ycontour;
  int nfound_contour=0,npts_contour=40;
  xcontour=new double[npts_contour];
  ycontour=new double[npts_contour];
  
  mninfo.x0=1.5;

  command = (char *) malloc (MAXLINE * sizeof (char));

  MNINIT(5,6,7);

  for (i = 0; i < 3; i++)
    MNPARM(i+1, pars[i].name, pars[i].value, pars[i].error, pars[i].min, pars[i].max, dummy);

  snprintf (command, MAXLINE, "MIGRAD");
  MNCOMD(mninfo.fcn, command, dummy, NULL);

  snprintf (command, MAXLINE, "SET ERR 0.25");
  MNCOMD(mninfo.fcn,command,dummy,NULL);
  MNCONT(mninfo.fcn,1,2,npts_contour,xcontour[0],ycontour[0],nfound_contour,0);

  snprintf (command, MAXLINE, "SET ERR 1.0");
  MNCOMD(mninfo.fcn,command,dummy,NULL);
  MNCONT(mninfo.fcn,1,2,npts_contour,xcontour[0],ycontour[0],nfound_contour,0);

  snprintf (command, MAXLINE, "SET ERR 2.0");
  MNCOMD(mninfo.fcn,command,dummy,NULL);
  MNCONT(mninfo.fcn,1,2,npts_contour,xcontour[0],ycontour[0],nfound_contour,0);

  exit (0);
}
